@@ -27,12 +27,16 @@
 #include "ovn-dirs.h"
 #include "ovn-nb-idl.h"
 #include "ovn-sb-idl.h"
+#include "ovsdb-idl.h"
 #include "socket-util.h"
+#include "stream.h"
 #include "svec.h"
 #include "unixctl.h"
 
 VLOG_DEFINE_THIS_MODULE(ovn_util);
 
+#define DEFAULT_PROBE_INTERVAL_MSEC 5000
+
 void ovn_conn_show(struct unixctl_conn *conn, int argc OVS_UNUSED,
                    const char *argv[] OVS_UNUSED, void *idl_)
 {
@@ -43,6 +47,24 @@ void ovn_conn_show(struct unixctl_conn *conn, int argc OVS_UNUSED,
         ovsdb_idl_is_connected(idl) ? "connected": "not connected");
 }
 
+/* Set inactivity probe interval for 'idl' and 'remote' to 'interval'.
+ * If 'interval' < 0 (no preference from daemon settings), set it to 5000ms;
+ * if 'remote' needs probing, disable otherwise.
+ * 'interval' value of 0 disables probing.
+ */
+void set_idl_probe_interval(struct ovsdb_idl *idl, const char *remote,
+                            int interval)
+{
+    if (interval < 0) {
+        interval = (remote && !stream_or_pstream_needs_probes(remote)
+                    ? 0 : DEFAULT_PROBE_INTERVAL_MSEC);
+    } else if (interval > 0 && interval < 1000) {
+        interval = 1000;
+    }
+
+    ovsdb_idl_set_probe_interval(idl, interval);
+}
+
 static void
 add_ipv4_netaddr(struct lport_addresses *laddrs, ovs_be32 addr,
                  unsigned int plen)
@@ -463,9 +485,9 @@ split_addresses(const char *addresses, struct svec *ipv4_addrs,
  *
  * It is the caller's responsibility to free the allocated memory. */
 char *
-alloc_nat_zone_key(const struct uuid *key, const char *type)
+alloc_nat_zone_key(const char *name, const char *type)
 {
-    return xasprintf(UUID_FMT"_%s", UUID_ARGS(key), type);
+    return xasprintf("%s_%s", name, type);
 }
 
 const char *
@@ -698,6 +720,21 @@ ovn_allocate_tnlid(struct hmap *set, const char *name, uint32_t min,
     return 0;
 }
 
+bool
+ovn_free_tnlid(struct hmap *tnlids, uint32_t tnlid)
+{
+    uint32_t hash = hash_int(tnlid, 0);
+    struct tnlid_node *node;
+    HMAP_FOR_EACH_IN_BUCKET (node, hmap_node, hash, tnlids) {
+        if (node->tnlid == tnlid) {
+            hmap_remove(tnlids, &node->hmap_node);
+            free(node);
+            return true;
+        }
+    }
+    return false;
+}
+
 char *
 ovn_chassis_redirect_name(const char *port_name)
 {
@@ -723,6 +760,18 @@ ip46_parse_cidr(const char *str, struct in6_addr *prefix, unsigned int *plen)
     return false;
 }
 
+bool
+ip46_parse(const char *ip_str, struct in6_addr *ip)
+{
+    ovs_be32 ipv4;
+    if (ip_parse(ip_str, &ipv4)) {
+        *ip = in6_addr_mapped_ipv4(ipv4);
+        return true;
+    }
+
+    return ipv6_parse(ip_str, ip);
+}
+
 /* The caller must free the returned string. */
 char *
 normalize_ipv4_prefix(ovs_be32 ipv4, unsigned int plen)
@@ -1092,3 +1141,164 @@ void flow_collector_ids_clear(struct flow_collector_ids *ids)
     flow_collector_ids_destroy(ids);
     flow_collector_ids_init(ids);
 }
+
+char *
+encode_fqdn_string(const char *fqdn, size_t *len)
+{
+
+    size_t domain_len = strlen(fqdn);
+    *len = domain_len + 2;
+    char *encoded = xzalloc(*len);
+
+    int8_t label_len = 0;
+    for (size_t i = 0; i < domain_len; i++) {
+        if (fqdn[i] == '.') {
+            encoded[i - label_len] = label_len;
+            label_len = 0;
+        } else {
+            encoded[i + 1] = fqdn[i];
+            label_len++;
+        }
+    }
+
+    /* This is required for the last label if it doesn't end with '.' */
+    if (label_len) {
+        encoded[domain_len - label_len] = label_len;
+    }
+
+    return encoded;
+}
+
+static bool
+is_lport_vif(const struct sbrec_port_binding *pb)
+{
+    return !pb->type[0];
+}
+
+enum en_lport_type
+get_lport_type(const struct sbrec_port_binding *pb)
+{
+    if (is_lport_vif(pb)) {
+        if (pb->parent_port && pb->parent_port[0]) {
+            return LP_CONTAINER;
+        }
+        return LP_VIF;
+    } else if (!strcmp(pb->type, "patch")) {
+        return LP_PATCH;
+    } else if (!strcmp(pb->type, "chassisredirect")) {
+        return LP_CHASSISREDIRECT;
+    } else if (!strcmp(pb->type, "l3gateway")) {
+        return LP_L3GATEWAY;
+    } else if (!strcmp(pb->type, "localnet")) {
+        return LP_LOCALNET;
+    } else if (!strcmp(pb->type, "localport")) {
+        return LP_LOCALPORT;
+    } else if (!strcmp(pb->type, "l2gateway")) {
+        return LP_L2GATEWAY;
+    } else if (!strcmp(pb->type, "virtual")) {
+        return LP_VIRTUAL;
+    } else if (!strcmp(pb->type, "external")) {
+        return LP_EXTERNAL;
+    } else if (!strcmp(pb->type, "remote")) {
+        return LP_REMOTE;
+    } else if (!strcmp(pb->type, "vtep")) {
+        return LP_VTEP;
+    }
+
+    return LP_UNKNOWN;
+}
+
+char *
+get_lport_type_str(enum en_lport_type lport_type)
+{
+    switch (lport_type) {
+    case LP_VIF:
+        return "VIF";
+    case LP_CONTAINER:
+        return "CONTAINER";
+    case LP_VIRTUAL:
+        return "VIRTUAL";
+    case LP_PATCH:
+        return "PATCH";
+    case LP_CHASSISREDIRECT:
+        return "CHASSISREDIRECT";
+    case LP_L3GATEWAY:
+        return "L3GATEWAY";
+    case LP_LOCALNET:
+        return "LOCALNET";
+    case LP_LOCALPORT:
+        return "LOCALPORT";
+    case LP_L2GATEWAY:
+        return "L2GATEWAY";
+    case LP_EXTERNAL:
+        return "EXTERNAL";
+    case LP_REMOTE:
+        return "REMOTE";
+    case LP_VTEP:
+        return "VTEP";
+    case LP_UNKNOWN:
+        return "UNKNOWN";
+    }
+
+    OVS_NOT_REACHED();
+}
+
+bool
+is_pb_router_type(const struct sbrec_port_binding *pb)
+{
+    enum en_lport_type lport_type = get_lport_type(pb);
+
+    switch (lport_type) {
+    case LP_PATCH:
+    case LP_CHASSISREDIRECT:
+    case LP_L3GATEWAY:
+    case LP_L2GATEWAY:
+        return true;
+
+    case LP_VIF:
+    case LP_CONTAINER:
+    case LP_VIRTUAL:
+    case LP_LOCALNET:
+    case LP_LOCALPORT:
+    case LP_REMOTE:
+    case LP_VTEP:
+    case LP_EXTERNAL:
+    case LP_UNKNOWN:
+        return false;
+    }
+
+    return false;
+}
+
+void
+sorted_array_apply_diff(const struct sorted_array *a1,
+                        const struct sorted_array *a2,
+                        void (*apply_callback)(const void *arg,
+                                               const char *item,
+                                               bool add),
+                        const void *arg)
+{
+    size_t idx1, idx2;
+
+    for (idx1 = idx2 = 0; idx1 < a1->n && idx2 < a2->n;) {
+        int cmp = strcmp(a1->arr[idx1], a2->arr[idx2]);
+        if (cmp < 0) {
+            apply_callback(arg, a1->arr[idx1], true);
+            idx1++;
+        } else if (cmp > 0) {
+            apply_callback(arg, a2->arr[idx2], false);
+            idx2++;
+        } else {
+            idx1++;
+            idx2++;
+        }
+    }
+
+    for (; idx1 < a1->n; idx1++) {
+        apply_callback(arg, a1->arr[idx1], true);
+    }
+
+    for (; idx2 < a2->n; idx2++) {
+        apply_callback(arg, a2->arr[idx2], false);
+    }
+}
