@@ -30,5 +30,7 @@
 #define LFLOWS_IGMP_STOPWATCH_NAME "lflows_igmp"
 #define LFLOWS_DP_GROUPS_STOPWATCH_NAME "lflows_dp_groups"
 #define LFLOWS_TO_SB_STOPWATCH_NAME "lflows_to_sb"
+#define PORT_GROUP_RUN_STOPWATCH_NAME "port_group_run"
+#define SYNC_METERS_RUN_STOPWATCH_NAME "sync_meters_run"
 
 #endif
