@@ -72,11 +72,6 @@ static const char *ssl_private_key_file;
 static const char *ssl_certificate_file;
 static const char *ssl_ca_cert_file;
 
-/* Default probe interval for NB and SB DB connections. */
-#define DEFAULT_PROBE_INTERVAL_MSEC 5000
-static int northd_probe_interval_nb = 0;
-static int northd_probe_interval_sb = 0;
-
 static const char *rbac_chassis_auth[] =
     {"name"};
 static const char *rbac_chassis_update[] =
@@ -102,7 +97,7 @@ static const char *rbac_controller_event_update[] =
 static const char *rbac_fdb_auth[] =
     {""};
 static const char *rbac_fdb_update[] =
-    {"dp_key", "mac", "port_key"};
+    {"dp_key", "mac", "port_key", "timestamp"};
 
 static const char *rbac_port_binding_auth[] =
     {""};
@@ -277,7 +272,10 @@ static struct gen_opts_map supported_dhcpv6_opts[] = {
     DHCPV6_OPT_IA_ADDR,
     DHCPV6_OPT_SERVER_ID,
     DHCPV6_OPT_DOMAIN_SEARCH,
-    DHCPV6_OPT_DNS_SERVER
+    DHCPV6_OPT_DNS_SERVER,
+    DHCPV6_OPT_BOOTFILE_NAME,
+    DHCPV6_OPT_BOOTFILE_NAME_ALT,
+    DHCPV6_OPT_FQDN,
 };
 
 static bool
@@ -687,22 +685,9 @@ update_ssl_config(void)
     }
 }
 
-static int
-get_probe_interval(const char *db, const struct nbrec_nb_global *nb)
-{
-    int default_interval = (db && !stream_or_pstream_needs_probes(db)
-                            ? 0 : DEFAULT_PROBE_INTERVAL_MSEC);
-    int interval = smap_get_int(&nb->options,
-                                "northd_probe_interval", default_interval);
-
-    if (interval > 0 && interval < 1000) {
-        interval = 1000;
-    }
-    return interval;
-}
-
 static struct ovsdb_idl_txn *
-run_idl_loop(struct ovsdb_idl_loop *idl_loop, const char *name)
+run_idl_loop(struct ovsdb_idl_loop *idl_loop, const char *name,
+             uint64_t *idl_duration)
 {
     unsigned long long duration, start = time_msec();
     unsigned int seqno = UINT_MAX;
@@ -710,9 +695,9 @@ run_idl_loop(struct ovsdb_idl_loop *idl_loop, const char *name)
     int n = 0;
 
     /* Accumulate database changes as long as there are some,
-     * but no longer than half a second. */
+     * but no longer than "IDL_LOOP_MAX_DURATION_MS". */
     while (seqno != ovsdb_idl_get_seqno(idl_loop->idl)
-           && time_msec() - start < 500) {
+           && time_msec() - start < IDL_LOOP_MAX_DURATION_MS) {
         seqno = ovsdb_idl_get_seqno(idl_loop->idl);
         ovsdb_idl_run(idl_loop->idl);
         n++;
@@ -721,13 +706,15 @@ run_idl_loop(struct ovsdb_idl_loop *idl_loop, const char *name)
     txn = ovsdb_idl_loop_run(idl_loop);
 
     duration = time_msec() - start;
+    *idl_duration = duration;
     /* ovsdb_idl_run() is called at least 2 times.  Once directly and
      * once in the ovsdb_idl_loop_run().  n > 2 means that we received
      * data on at least 2 subsequent calls. */
     if (n > 2 || duration > 100) {
-        VLOG(duration > 500 ? VLL_INFO : VLL_DBG,
+        VLOG(duration > IDL_LOOP_MAX_DURATION_MS ? VLL_INFO : VLL_DBG,
              "%s IDL run: %d iterations in %lld ms", name, n + 1, duration);
     }
+
     return txn;
 }
 
@@ -778,7 +765,7 @@ main(int argc, char *argv[])
     service_start(&argc, &argv);
     parse_options(argc, argv, &state.paused, &n_threads);
 
-    daemonize_start(false);
+    daemonize_start(false, false);
 
     char *abs_unixctl_path = get_abs_unix_ctl_path(unixctl_path);
     retval = unixctl_server_create(abs_unixctl_path, &unixctl);
@@ -838,10 +825,29 @@ main(int argc, char *argv[])
     ovsdb_idl_track_add_all(ovnsb_idl_loop.idl);
     ovsdb_idl_set_write_changed_only_all(ovnsb_idl_loop.idl, true);
 
+    /* Omit unused columns. */
+    ovsdb_idl_omit(ovnsb_idl_loop.idl, &sbrec_sb_global_col_connections);
+    ovsdb_idl_omit(ovnsb_idl_loop.idl, &sbrec_sb_global_col_ssl);
+
     /* Disable alerting for pure write-only columns. */
     ovsdb_idl_omit_alert(ovnsb_idl_loop.idl, &sbrec_sb_global_col_nb_cfg);
     ovsdb_idl_omit_alert(ovnsb_idl_loop.idl, &sbrec_address_set_col_name);
     ovsdb_idl_omit_alert(ovnsb_idl_loop.idl, &sbrec_address_set_col_addresses);
+    for (size_t i = 0; i < SBREC_LOGICAL_FLOW_N_COLUMNS; i++) {
+        ovsdb_idl_omit_alert(ovnsb_idl_loop.idl,
+                             &sbrec_logical_flow_columns[i]);
+    }
+    for (size_t i = 0; i < SBREC_MULTICAST_GROUP_N_COLUMNS; i++) {
+        ovsdb_idl_omit_alert(ovnsb_idl_loop.idl,
+                             &sbrec_multicast_group_columns[i]);
+    }
+    for (size_t i = 0; i < SBREC_METER_N_COLUMNS; i++) {
+        ovsdb_idl_omit_alert(ovnsb_idl_loop.idl, &sbrec_meter_columns[i]);
+    }
+    for (size_t i = 0; i < SBREC_PORT_GROUP_N_COLUMNS; i++) {
+        ovsdb_idl_omit_alert(ovnsb_idl_loop.idl,
+                             &sbrec_port_group_columns[i]);
+    }
 
     unixctl_command_register("sb-connection-status", "", 0, 0,
                              ovn_conn_show, ovnsb_idl_loop.idl);
@@ -874,7 +880,8 @@ main(int argc, char *argv[])
     /* Main loop. */
     exiting = false;
 
-    bool recompute = false;
+    struct northd_engine_context eng_ctx = {};
+
     while (!exiting) {
         update_ssl_config();
         memory_run();
@@ -887,6 +894,7 @@ main(int argc, char *argv[])
             simap_destroy(&usage);
         }
 
+        bool clear_idl_track = true;
         if (!state.paused) {
             if (!ovsdb_idl_has_lock(ovnsb_idl_loop.idl) &&
                 !ovsdb_idl_is_lock_contended(ovnsb_idl_loop.idl))
@@ -899,26 +907,28 @@ main(int argc, char *argv[])
                 ovsdb_idl_set_lock(ovnsb_idl_loop.idl, "ovn_northd");
             }
 
-            struct ovsdb_idl_txn *ovnnb_txn = run_idl_loop(&ovnnb_idl_loop,
-                                                           "OVN_Northbound");
+            struct ovsdb_idl_txn *ovnnb_txn =
+                    run_idl_loop(&ovnnb_idl_loop, "OVN_Northbound",
+                                 &eng_ctx.nb_idl_duration_ms);
             unsigned int new_ovnnb_cond_seqno =
                         ovsdb_idl_get_condition_seqno(ovnnb_idl_loop.idl);
             if (new_ovnnb_cond_seqno != ovnnb_cond_seqno) {
                 if (!new_ovnnb_cond_seqno) {
                     VLOG_INFO("OVN NB IDL reconnected, force recompute.");
-                    recompute = true;
+                    eng_ctx.recompute = true;
                 }
                 ovnnb_cond_seqno = new_ovnnb_cond_seqno;
             }
 
-            struct ovsdb_idl_txn *ovnsb_txn = run_idl_loop(&ovnsb_idl_loop,
-                                                           "OVN_Southbound");
+            struct ovsdb_idl_txn *ovnsb_txn =
+                    run_idl_loop(&ovnsb_idl_loop, "OVN_Southbound",
+                                 &eng_ctx.sb_idl_duration_ms);
             unsigned int new_ovnsb_cond_seqno =
                         ovsdb_idl_get_condition_seqno(ovnsb_idl_loop.idl);
             if (new_ovnsb_cond_seqno != ovnsb_cond_seqno) {
                 if (!new_ovnsb_cond_seqno) {
                     VLOG_INFO("OVN SB IDL reconnected, force recompute.");
-                    recompute = true;
+                    eng_ctx.recompute = true;
                 }
                 ovnsb_cond_seqno = new_ovnsb_cond_seqno;
             }
@@ -936,25 +946,27 @@ main(int argc, char *argv[])
             }
 
             if (ovsdb_idl_has_lock(ovnsb_idl_loop.idl)) {
-                int64_t loop_start_time = time_wall_msec();
-                bool activity = inc_proc_northd_run(ovnnb_txn, ovnsb_txn,
-                                                    recompute);
-                recompute = false;
-                if (ovnsb_txn) {
+                bool activity = false;
+                if (ovnnb_txn && ovnsb_txn &&
+                    inc_proc_northd_can_run(&eng_ctx)) {
+                    int64_t loop_start_time = time_wall_msec();
+                    activity = inc_proc_northd_run(ovnnb_txn, ovnsb_txn,
+                                                   &eng_ctx);
+                    eng_ctx.recompute = false;
                     check_and_add_supported_dhcp_opts_to_sb_db(
                                  ovnsb_txn, ovnsb_idl_loop.idl);
                     check_and_add_supported_dhcpv6_opts_to_sb_db(
                                  ovnsb_txn, ovnsb_idl_loop.idl);
                     check_and_update_rbac(
                                  ovnsb_txn, ovnsb_idl_loop.idl);
-                }
 
-                if (ovnnb_txn && ovnsb_txn) {
                     update_sequence_numbers(loop_start_time,
                                             ovnnb_idl_loop.idl,
                                             ovnsb_idl_loop.idl,
                                             ovnnb_txn, ovnsb_txn,
                                             &ovnsb_idl_loop);
+                } else if (!eng_ctx.recompute) {
+                    clear_idl_track = false;
                 }
 
                 /* If there are any errors, we force a full recompute in order
@@ -962,13 +974,13 @@ main(int argc, char *argv[])
                 if (!ovsdb_idl_loop_commit_and_wait(&ovnnb_idl_loop)) {
                     VLOG_INFO("OVNNB commit failed, "
                               "force recompute next time.");
-                    recompute = true;
+                    eng_ctx.recompute = true;
                 }
 
                 if (!ovsdb_idl_loop_commit_and_wait(&ovnsb_idl_loop)) {
                     VLOG_INFO("OVNSB commit failed, "
                               "force recompute next time.");
-                    recompute = true;
+                    eng_ctx.recompute = true;
                 }
                 run_memory_trimmer(ovnnb_idl_loop.idl, activity);
             } else {
@@ -977,7 +989,7 @@ main(int argc, char *argv[])
                 ovsdb_idl_loop_commit_and_wait(&ovnsb_idl_loop);
 
                 /* Force a full recompute next time we become active. */
-                recompute = true;
+                eng_ctx.recompute = true;
             }
         } else {
             /* ovn-northd is paused
@@ -1001,11 +1013,13 @@ main(int argc, char *argv[])
             ovsdb_idl_wait(ovnsb_idl_loop.idl);
 
             /* Force a full recompute next time we become active. */
-            recompute = true;
+            eng_ctx.recompute = true;
         }
 
-        ovsdb_idl_track_clear(ovnnb_idl_loop.idl);
-        ovsdb_idl_track_clear(ovnsb_idl_loop.idl);
+        if (clear_idl_track) {
+            ovsdb_idl_track_clear(ovnnb_idl_loop.idl);
+            ovsdb_idl_track_clear(ovnsb_idl_loop.idl);
+        }
 
         unixctl_server_run(unixctl);
         unixctl_server_wait(unixctl);
@@ -1017,14 +1031,16 @@ main(int argc, char *argv[])
         const struct nbrec_nb_global *nb =
             nbrec_nb_global_first(ovnnb_idl_loop.idl);
         /* Update the probe interval. */
+        int interval = -1;
         if (nb) {
-            northd_probe_interval_nb = get_probe_interval(ovnnb_db, nb);
-            northd_probe_interval_sb = get_probe_interval(ovnsb_db, nb);
+            interval = smap_get_int(&nb->options, "northd_probe_interval",
+                                    interval);
+            eng_ctx.backoff_ms =
+                    smap_get_uint(&nb->options, "northd-backoff-interval-ms",
+                                  0);
         }
-        ovsdb_idl_set_probe_interval(ovnnb_idl_loop.idl,
-                                     northd_probe_interval_nb);
-        ovsdb_idl_set_probe_interval(ovnsb_idl_loop.idl,
-                                     northd_probe_interval_sb);
+        set_idl_probe_interval(ovnnb_idl_loop.idl, ovnnb_db, interval);
+        set_idl_probe_interval(ovnsb_idl_loop.idl, ovnsb_db, interval);
 
         if (reset_ovnsb_idl_min_index) {
             VLOG_INFO("Resetting southbound database cluster state");
